package {
	import dto.ImageDTO;
	
	import flash.events.Event;
	import flash.net.FileFilter;
	import flash.net.FileReference;
	import flash.net.URLRequest;
	import flash.net.navigateToURL;
	import flash.utils.ByteArray;
	
	import mx.collections.ArrayCollection;
	import mx.controls.Alert;
	import mx.controls.advancedDataGridClasses.AdvancedDataGridColumn;
	import mx.core.Application;
	import mx.formatters.DateFormatter;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.remoting.RemoteObject;

	public class AlbumLogic extends Application {

		private var view:AlbumView = this as AlbumView;
		public function AlbumLogic() {
			super();
		}

		[Bindable]
		protected var imageFilename:String;
		[Bindable]
		protected var imageData:ByteArray;

		/* Select Image File */
		private var imageFile:FileReference;
		protected function selectImage():void {
			imageFile = new FileReference()
			imageFile.addEventListener(Event.SELECT, selectHandler);
			imageFile.addEventListener(Event.COMPLETE, completeHandler);
			imageFile.browse([new FileFilter("イメージ(*.jpg)", "*.jpg")]);
		}
		private function selectHandler(event:Event):void {
			var file:FileReference = FileReference(event.target);
            file.load();
		}
		private function completeHandler(event:Event):void {
			var file:FileReference = FileReference(event.target);
			if (file.data.length > 1000000) {
				Alert.show("最大1Mまでのファイルしか読み込めません", "Error");
				return;
			}
			var image:ByteArray = new ByteArray();
			file.data.readBytes(image, 0, file.data.length);
			this.imageFilename = file.name;
			this.imageData = image;
			this.imageFile = null;
		}


		/* Add Image */
		protected function add():void {
			var image:ImageDTO = new ImageDTO();
			image.image = this.imageData;
			image.title = view.formTitle.text;
			image.author = view.formAuthor.text;

			this.imageFilename = null;
			this.imageData = null;

			var albumService:RemoteObject = new RemoteObject("albumService");
			albumService.addEventListener(ResultEvent.RESULT, addHandler);
			albumService.addEventListener(FaultEvent.FAULT, serviceFaultHandler);

			albumService.add(image);
			busyAdd();
		}
		private function addHandler(event:ResultEvent):void {
			var id:String = event.result as String;
			Alert.show("Added: " + id);
			busyRemove();
		}
		
		/* Show Image */
		protected function showImageHandler(event:Event):void {
			var imageId:String = this.view.list.selectedItem["imageId"] as String;			
			navigateToURL(new URLRequest("/image/" + imageId), "image");
		}

		/* Search Image */
		protected function search():void {
			var albumService:RemoteObject = new RemoteObject("albumService");
			albumService.addEventListener(ResultEvent.RESULT, searchHandler);
			albumService.addEventListener(FaultEvent.FAULT, serviceFaultHandler);
			albumService.search();
			busyAdd();
		}
		private function searchHandler(event:ResultEvent):void {
			var result:ArrayCollection = event.result as ArrayCollection;
			view.list.dataProvider = result;
			busyRemove();
		}

		/* Common Fault Handler */
		private function serviceFaultHandler(event:FaultEvent):void {
			busyRemove();
			if (event.fault.rootCause != null && event.fault.rootCause.hasOwnProperty("message")) {
				Alert.show(event.fault.rootCause.message);
			} else {
				Alert.show(event.message.toString());
			}
		}
		
		/* Label Function */
		private var dateFormatter:DateFormatter;
		protected function labelDate(item:Object, column:AdvancedDataGridColumn):String {
			var date:Date = item[column.dataField] as Date;
			if (date == null) return "";

			if (this.dateFormatter == null) {
				this.dateFormatter = new DateFormatter();
				this.dateFormatter.formatString  = "YYYY/MM/DD J:NN:SS";
			}
			return this.dateFormatter.format(date);
		}

		/* Busy Icon Controll */
		private var busys:int = 0;
		private function busyAdd():void {
			if (this.busys == 0) {
				cursorManager.setBusyCursor();
			}
			this.busys++;
		}
		private function busyRemove():void {
			this.busys--;
			if (this.busys < 0) this.busys = 0;
			if (this.busys == 0) {
				cursorManager.removeBusyCursor();
			}
		}
	}
}
